"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthRouter = void 0;
const express_1 = require("express");
const express_session_1 = __importDefault(require("express-session"));
class AuthRouter {
    constructor(container) {
        var _a;
        this.container = container;
        this.router = (0, express_1.Router)();
        this.authService = container.resolve("authService");
        const config = container.resolve("configModule");
        this.cookieSecret = ((_a = config.projectConfig) === null || _a === void 0 ? void 0 : _a.cookie_secret) || "default_secret";
        this.setupRoutes();
    }
    setupRoutes() {
        // Configure session middleware
        this.router.use((0, express_session_1.default)({
            secret: this.cookieSecret,
            resave: false,
            saveUninitialized: false,
            cookie: {
                secure: process.env.NODE_ENV === "production",
                maxAge: 24 * 60 * 60 * 1000 // 24 hours
            }
        }));
        // Fix routes by using proper router methods
        this.router.get("/auth/:provider", (req, res, next) => {
            // Skip if it's our dedicated Google route
            if (req.params.provider === "google" && req.path.startsWith("/store/")) {
                return next();
            }
            this.handleAuth(req, res);
        });
        this.router.get("/auth/:provider/callback", (req, res, next) => {
            // Skip if it's our dedicated Google callback route
            if (req.params.provider === "google" && req.path.startsWith("/store/")) {
                return next();
            }
            this.handleCallback(req, res);
        });
        this.router.get("/auth/success", (req, res) => this.handleSuccess(req, res));
        this.router.get("/auth/error", (req, res) => this.handleError(req, res));
    }
    async handleAuth(req, res) {
        const { provider } = req.params;
        try {
            // Just initiate authentication - do not end the response
            // This allows Passport to redirect to the OAuth provider
            await this.authService.authenticate(provider, req);
            // Remove res.end() to allow redirect to happen
        }
        catch (error) {
            req.session.authError = error;
            res.redirect("/auth/error");
        }
    }
    async handleCallback(req, res) {
        const { provider } = req.params;
        try {
            const result = await this.authService.authenticate(provider, req);
            if (!(result === null || result === void 0 ? void 0 : result.profile)) {
                throw new Error("Authentication failed: No profile returned");
            }
            req.session.authProfile = result.profile;
            res.redirect("/auth/success");
        }
        catch (error) {
            req.session.authError = error;
            res.redirect("/auth/error");
        }
    }
    handleSuccess(req, res) {
        const profile = req.session.authProfile;
        if (!profile) {
            return res.redirect("/auth/error");
        }
        res.json({ success: true, profile });
    }
    handleError(req, res) {
        const error = req.session.authError;
        res.status(401).json({
            success: false,
            error: (error === null || error === void 0 ? void 0 : error.message) || "Authentication failed"
        });
    }
    getRouter() {
        return this.router;
    }
}
exports.AuthRouter = AuthRouter;
//# sourceMappingURL=auth.router.js.map