"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MEDUSA_AUTH_CONFIG_KEY = void 0;
exports.loadConfig = loadConfig;
exports.getConfig = getConfig;
const config_1 = require("../validators/config");
exports.MEDUSA_AUTH_CONFIG_KEY = "auth_config";
/**
 * Partial default configuration values.
 * Note: Required values like secrets must be provided by the user.
 */
const defaultConfig = {
    redirect: {
        success: "/",
        failure: "/auth/login",
    },
};
/**
 * Default session configuration without secrets
 */
const defaultSessionConfig = {
    name: "medusa_auth_session",
    ttl: 24 * 60 * 60 * 1000, // 24 hours
    domain: undefined,
};
/**
 * Loads and validates the auth configuration
 */
async function loadConfig(container, config) {
    try {
        if (!config) {
            throw new Error("Auth configuration is required");
        }
        // Merge provided config with defaults, ensuring required fields are preserved
        const mergedConfig = {
            ...defaultConfig,
            ...config,
            session: config.session && {
                ...defaultSessionConfig,
                ...config.session,
            },
            jwt: config.jwt,
            redirect: {
                ...defaultConfig.redirect,
                ...config === null || config === void 0 ? void 0 : config.redirect,
            },
            // Providers must be provided by the user
            providers: config.providers,
        };
        // Validate the configuration
        config_1.ConfigValidator.validate(mergedConfig);
        // Register the config in the container
        container.register(exports.MEDUSA_AUTH_CONFIG_KEY, {
            resolve: () => mergedConfig,
        });
    }
    catch (error) {
        // Enhance error message for better debugging
        throw new Error(`Failed to load auth configuration: ${error.message}`);
    }
}
/**
 * Retrieves the auth configuration from the container
 */
function getConfig(container) {
    try {
        const configResolver = container.resolve(exports.MEDUSA_AUTH_CONFIG_KEY);
        if (!configResolver) {
            throw new Error("Auth configuration not found in container");
        }
        return configResolver.resolve();
    }
    catch (error) {
        throw new Error(`Failed to retrieve auth configuration: ${error.message}`);
    }
}
//# sourceMappingURL=config.js.map