"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FacebookOAuth2Strategy = void 0;
const oauth2_strategy_1 = require("./oauth2-strategy");
const passport_facebook_1 = require("passport-facebook");
class FacebookOAuth2Strategy extends oauth2_strategy_1.OAuth2BaseStrategy {
    constructor(container, config) {
        super(container, "facebook", {
            clientID: config.clientID,
            clientSecret: config.clientSecret,
            callbackURL: config.callbackURL,
            graphApiVersion: config.graphApiVersion || "v12.0"
        });
        this.graphApiVersion = this.getOAuth2Config().graphApiVersion || "v12.0";
        const strategyConfig = {
            clientID: this.getOAuth2Config().clientID,
            clientSecret: this.getOAuth2Config().clientSecret,
            callbackURL: this.getOAuth2Config().callbackURL,
            profileFields: ["id", "emails", "name", "displayName"],
            graphAPIVersion: this.graphApiVersion
        };
        this.strategy = new passport_facebook_1.Strategy(strategyConfig, this.validateCallback.bind(this));
    }
    async validateCallback(accessToken, refreshToken, profile, done) {
        var _a, _b, _c, _d;
        try {
            const transformedProfile = {
                id: profile.id,
                email: ((_b = (_a = profile.emails) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.value) || "",
                first_name: ((_c = profile.name) === null || _c === void 0 ? void 0 : _c.givenName) || "",
                last_name: ((_d = profile.name) === null || _d === void 0 ? void 0 : _d.familyName) || "",
                displayName: profile.displayName || "",
                provider: this.provider,
                metadata: {
                    raw: profile,
                    accessToken,
                    refreshToken
                }
            };
            const authProfile = this.transformToAuthProfile(transformedProfile);
            done(null, authProfile);
        }
        catch (error) {
            done(error);
        }
    }
    authenticateRequest(req) {
        this.strategy.authenticate(req, { session: false });
    }
    async transformProfile(profile) {
        return {
            id: profile.id,
            email: profile.email,
            first_name: profile.firstName,
            last_name: profile.lastName,
            displayName: profile.displayName,
            provider: profile.provider,
            metadata: profile.metadata
        };
    }
    async authenticate(reqOrData) {
        if (reqOrData instanceof Request) {
            const query = reqOrData.query;
            const code = query === null || query === void 0 ? void 0 : query.code;
            if (!code) {
                throw new Error("Authorization code is required");
            }
            const accessToken = await this.getAccessToken(code);
            const fbProfile = await this.getProfile(accessToken);
            const transformedProfile = await this.transformProfile(fbProfile);
            return {
                provider: this.provider,
                accessToken,
                profile: this.transformToAuthProfile(transformedProfile)
            };
        }
        // Handle direct AuthEventData
        if (!this.isAuthEventData(reqOrData)) {
            throw new Error("Invalid auth data format");
        }
        return reqOrData;
    }
    isAuthEventData(data) {
        return data && typeof data === 'object' && 'provider' in data;
    }
    async getProfile(accessToken) {
        const fields = ["id", "email", "first_name", "last_name", "name"].join(",");
        const url = `https://graph.facebook.com/${this.graphApiVersion}/me?fields=${fields}&access_token=${accessToken}`;
        const response = await fetch(url);
        if (!response.ok) {
            throw new Error("Failed to fetch Facebook profile");
        }
        const data = await response.json();
        const authProfile = {
            id: data.id,
            email: data.email || "",
            firstName: data.first_name || "",
            lastName: data.last_name || "",
            displayName: data.name || "",
            provider: this.provider,
            metadata: {
                raw: data
            }
        };
        return authProfile;
    }
    async getAccessToken(code) {
        const config = this.getOAuth2Config();
        const params = new URLSearchParams({
            client_id: config.clientID,
            client_secret: config.clientSecret,
            redirect_uri: config.callbackURL,
            code
        });
        const url = `https://graph.facebook.com/${this.graphApiVersion}/oauth/access_token?${params.toString()}`;
        const response = await fetch(url);
        if (!response.ok) {
            throw new Error("Failed to get access token from Facebook");
        }
        const data = await response.json();
        if (!data.access_token) {
            throw new Error("No access token received from Facebook");
        }
        return data.access_token;
    }
    async afterAuth(data) {
        return data;
    }
    async onSuccess(data) {
        return {
            ...data,
            metadata: {
                ...data.metadata,
                status: "success"
            }
        };
    }
    async onError(data) {
        return {
            ...data,
            metadata: {
                ...data.metadata,
                status: "error"
            }
        };
    }
}
exports.FacebookOAuth2Strategy = FacebookOAuth2Strategy;
//# sourceMappingURL=facebook-strategy.js.map