"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitHubOAuth2Strategy = void 0;
const oauth2_strategy_1 = require("./oauth2-strategy");
const passport_github2_1 = require("passport-github2");
class GitHubOAuth2Strategy extends oauth2_strategy_1.OAuth2BaseStrategy {
    constructor(container, config) {
        super(container, "github", {
            clientID: config.clientID,
            clientSecret: config.clientSecret,
            callbackURL: config.callbackURL,
            scope: config.scope || ["user:email"]
        });
        const strategyConfig = {
            clientID: this.getOAuth2Config().clientID,
            clientSecret: this.getOAuth2Config().clientSecret,
            callbackURL: this.getOAuth2Config().callbackURL,
            scope: this.getOAuth2Config().scope
        };
        this.strategy = new passport_github2_1.Strategy(strategyConfig, this.validateCallback.bind(this));
    }
    async validateCallback(accessToken, refreshToken, profile, done) {
        var _a, _b, _c, _d;
        try {
            const transformedProfile = {
                id: profile.id,
                email: ((_b = (_a = profile.emails) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.value) || "",
                first_name: ((_c = profile.displayName) === null || _c === void 0 ? void 0 : _c.split(" ")[0]) || "",
                last_name: ((_d = profile.displayName) === null || _d === void 0 ? void 0 : _d.split(" ").slice(1).join(" ")) || "",
                displayName: profile.displayName || "",
                provider: this.provider,
                metadata: {
                    raw: profile,
                    accessToken,
                    refreshToken
                }
            };
            const authProfile = this.transformToAuthProfile(transformedProfile);
            done(null, authProfile);
        }
        catch (error) {
            done(error);
        }
    }
    authenticateRequest(req) {
        this.strategy.authenticate(req, { session: false });
    }
    async transformProfile(profile) {
        return {
            id: profile.id,
            email: profile.email,
            first_name: profile.firstName,
            last_name: profile.lastName,
            displayName: profile.displayName,
            provider: profile.provider,
            metadata: profile.metadata
        };
    }
    async authenticate(reqOrData) {
        if (reqOrData instanceof Request) {
            const query = reqOrData.query;
            const code = query === null || query === void 0 ? void 0 : query.code;
            if (!code) {
                throw new Error("Authorization code is required");
            }
            const accessToken = await this.getAccessToken(code);
            const githubProfile = await this.getProfile(accessToken);
            const transformedProfile = await this.transformProfile(githubProfile);
            return {
                provider: this.provider,
                accessToken,
                profile: this.transformToAuthProfile(transformedProfile)
            };
        }
        // Handle direct AuthEventData
        if (!this.isAuthEventData(reqOrData)) {
            throw new Error("Invalid auth data format");
        }
        return reqOrData;
    }
    isAuthEventData(data) {
        return data && typeof data === 'object' && 'provider' in data;
    }
    async getProfile(accessToken) {
        var _a, _b;
        const url = "https://api.github.com/user";
        const headers = {
            Authorization: `token ${accessToken}`,
            "User-Agent": "Medusa Auth Plugin"
        };
        const response = await fetch(url, { headers });
        if (!response.ok) {
            throw new Error("Failed to fetch GitHub profile");
        }
        const data = await response.json();
        const authProfile = {
            id: data.id.toString(),
            email: data.email || "",
            firstName: ((_a = data.name) === null || _a === void 0 ? void 0 : _a.split(" ")[0]) || "",
            lastName: ((_b = data.name) === null || _b === void 0 ? void 0 : _b.split(" ").slice(1).join(" ")) || "",
            displayName: data.name || data.login || "",
            provider: this.provider,
            metadata: {
                raw: data
            }
        };
        return authProfile;
    }
    async getAccessToken(code) {
        const config = this.getOAuth2Config();
        const params = new URLSearchParams({
            client_id: config.clientID,
            client_secret: config.clientSecret,
            code,
            redirect_uri: config.callbackURL
        });
        const url = `https://github.com/login/oauth/access_token?${params.toString()}`;
        const response = await fetch(url, {
            method: "POST",
            headers: {
                Accept: "application/json"
            }
        });
        if (!response.ok) {
            throw new Error("Failed to get access token from GitHub");
        }
        const data = await response.json();
        if (!data.access_token) {
            throw new Error("No access token received from GitHub");
        }
        return data.access_token;
    }
    async afterAuth(data) {
        return data;
    }
    async onSuccess(data) {
        return {
            ...data,
            metadata: {
                ...data.metadata,
                status: "success"
            }
        };
    }
    async onError(data) {
        return {
            ...data,
            metadata: {
                ...data.metadata,
                status: "error"
            }
        };
    }
}
exports.GitHubOAuth2Strategy = GitHubOAuth2Strategy;
//# sourceMappingURL=github-strategy.js.map