"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GoogleOAuth2Strategy = void 0;
const oauth2_strategy_1 = require("./oauth2-strategy");
const passport_google_oauth20_1 = require("passport-google-oauth20");
class GoogleOAuth2Strategy extends oauth2_strategy_1.OAuth2BaseStrategy {
    constructor(container, config) {
        super(container, "google", {
            clientID: config.clientID,
            clientSecret: config.clientSecret,
            callbackURL: config.callbackURL,
            scope: config.scope || ["profile", "email"]
        });
        const strategyConfig = {
            clientID: this.getOAuth2Config().clientID,
            clientSecret: this.getOAuth2Config().clientSecret,
            callbackURL: this.getOAuth2Config().callbackURL,
            scope: this.getOAuth2Config().scope
        };
        this.strategy = new passport_google_oauth20_1.Strategy(strategyConfig, this.validateCallback.bind(this));
    }
    async validateCallback(accessToken, refreshToken, profile, done) {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        try {
            console.log("[GOOGLE STRATEGY] Raw profile from Google:", JSON.stringify(profile, null, 2));
            // Enhanced profile transformation with more fields
            const transformedProfile = {
                id: profile.id,
                email: ((_b = (_a = profile.emails) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.value) || "",
                first_name: ((_c = profile.name) === null || _c === void 0 ? void 0 : _c.givenName) || "",
                last_name: ((_d = profile.name) === null || _d === void 0 ? void 0 : _d.familyName) || "",
                displayName: profile.displayName || "",
                provider: this.provider,
                picture: ((_f = (_e = profile.photos) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.value) || "",
                locale: ((_g = profile._json) === null || _g === void 0 ? void 0 : _g.locale) || "",
                verified: ((_h = profile._json) === null || _h === void 0 ? void 0 : _h.email_verified) || false,
                metadata: {
                    raw: profile,
                    accessToken,
                    refreshToken,
                    _json: profile._json || {},
                    photos: profile.photos || [],
                    emails: profile.emails || [],
                    isAuthenticated: true,
                    authenticatedAt: new Date().toISOString()
                }
            };
            const authProfile = this.transformToAuthProfile(transformedProfile);
            done(null, authProfile);
        }
        catch (error) {
            console.error("[GOOGLE STRATEGY] Error in validateCallback:", error);
            done(error);
        }
    }
    authenticateRequest(req) {
        console.log("[GOOGLE STRATEGY] authenticateRequest called");
        try {
            const config = this.getOAuth2Config();
            // Get Express response from req
            const res = req.res;
            if (!res) {
                console.error("[GOOGLE STRATEGY] Response object not available on request");
                throw new Error("Response object not available on request");
            }
            // Build the authorization URL manually
            const authUrl = new URL("https://accounts.google.com/o/oauth2/v2/auth");
            authUrl.searchParams.append("client_id", config.clientID);
            authUrl.searchParams.append("redirect_uri", config.callbackURL);
            authUrl.searchParams.append("response_type", "code");
            authUrl.searchParams.append("scope", config.scope.join(" "));
            authUrl.searchParams.append("access_type", "offline");
            authUrl.searchParams.append("prompt", "consent");
            // Add state parameter for security
            const state = Math.random().toString(36).substring(2, 15);
            authUrl.searchParams.append("state", state);
            console.log("[GOOGLE STRATEGY] Redirecting to Google auth URL:", authUrl.toString());
            // Redirect using Express
            res.redirect(authUrl.toString());
        }
        catch (error) {
            console.error("[GOOGLE STRATEGY] Error in authenticateRequest:", error);
            throw error;
        }
    }
    async transformProfile(profile) {
        return {
            id: profile.id,
            email: profile.email,
            first_name: profile.firstName,
            last_name: profile.lastName,
            displayName: profile.displayName,
            provider: profile.provider,
            metadata: profile.metadata
        };
    }
    async authenticate(reqOrData) {
        console.log("[GOOGLE STRATEGY] authenticate called with:", typeof reqOrData);
        try {
            // Case 1: Express Request with query parameters
            if (reqOrData instanceof Request && 'query' in reqOrData && reqOrData.query) {
                const code = reqOrData.query['code'];
                if (code && typeof code === 'string') {
                    console.log("[GOOGLE STRATEGY] Processing from Express Request with query params");
                    return this.handleAuthenticationCode(code);
                }
            }
            // Case 2: Object with code property (from API routes)
            if (reqOrData && typeof reqOrData === 'object' && 'code' in reqOrData) {
                console.log("[GOOGLE STRATEGY] Processing from object with code");
                const code = reqOrData.code;
                return this.handleAuthenticationCode(code);
            }
            // Case 3: Already processed AuthEventData
            if (reqOrData && typeof reqOrData === 'object' && 'provider' in reqOrData && reqOrData.provider === this.provider) {
                console.log("[GOOGLE STRATEGY] Passing through existing AuthEventData");
                return reqOrData;
            }
            throw new Error("Invalid authentication request format");
        }
        catch (error) {
            console.error("[GOOGLE STRATEGY] Error in authenticate method:", error);
            throw error;
        }
    }
    // Helper method to process authentication code
    async handleAuthenticationCode(code) {
        if (!code) {
            throw new Error("Authorization code is required");
        }
        console.log("[GOOGLE STRATEGY] Getting access token with code");
        const accessToken = await this.getAccessToken(code);
        console.log("[GOOGLE STRATEGY] Getting profile with access token");
        const googleProfile = await this.getProfile(accessToken);
        console.log("[GOOGLE STRATEGY] Transforming profile");
        const transformedProfile = await this.transformProfile(googleProfile);
        return {
            provider: this.provider,
            accessToken,
            profile: this.transformToAuthProfile(transformedProfile)
        };
    }
    async getProfile(accessToken) {
        const url = "https://www.googleapis.com/oauth2/v3/userinfo";
        const headers = {
            Authorization: `Bearer ${accessToken}`
        };
        const response = await fetch(url, { headers });
        if (!response.ok) {
            throw new Error("Failed to fetch Google profile");
        }
        const data = await response.json();
        const authProfile = {
            id: data.sub,
            email: data.email || "",
            firstName: data.given_name || "",
            lastName: data.family_name || "",
            displayName: data.name || "",
            provider: this.provider,
            metadata: {
                raw: data
            }
        };
        return authProfile;
    }
    async getAccessToken(code) {
        const config = this.getOAuth2Config();
        const params = new URLSearchParams({
            client_id: config.clientID,
            client_secret: config.clientSecret,
            code,
            grant_type: "authorization_code",
            redirect_uri: config.callbackURL
        });
        const url = "https://oauth2.googleapis.com/token";
        const response = await fetch(url, {
            method: "POST",
            headers: {
                "Content-Type": "application/x-www-form-urlencoded"
            },
            body: params.toString()
        });
        if (!response.ok) {
            const errorData = await response.text();
            console.error("[GOOGLE STRATEGY] Token exchange error:", errorData);
            throw new Error(`Failed to get access token from Google: ${response.status} ${response.statusText}`);
        }
        const data = await response.json();
        if (!data.access_token) {
            throw new Error("No access token received from Google");
        }
        return data.access_token;
    }
    async afterAuth(data) {
        var _a, _b;
        console.log("[GOOGLE STRATEGY] afterAuth called");
        try {
            // This is where you would typically create or update a customer account
            // and generate a JWT token for authenticated sessions
            // Mock implementation for testing
            const result = {
                success: true,
                token: "mock_jwt_token_" + Date.now(),
                customer: {
                    id: "cus_" + Math.random().toString(36).substring(2, 10),
                    email: ((_a = data.profile) === null || _a === void 0 ? void 0 : _a.email) || "customer@example.com"
                },
                redirect: process.env.GOOGLE_AUTH_SUCCESS_REDIRECT ||
                    "/auth/success?email=" + encodeURIComponent(((_b = data.profile) === null || _b === void 0 ? void 0 : _b.email) || "")
            };
            console.log("[GOOGLE STRATEGY] afterAuth completed with success");
            return result;
        }
        catch (error) {
            console.error("[GOOGLE STRATEGY] Error in afterAuth:", error);
            return {
                success: false,
                error: error.message
            };
        }
    }
    async onSuccess(data) {
        return {
            ...data,
            metadata: {
                ...data.metadata,
                status: "success"
            }
        };
    }
    async onError(data) {
        return {
            ...data,
            metadata: {
                ...data.metadata,
                status: "error"
            }
        };
    }
    // Add this method to expose the OAuth2 config
    getOAuth2Config() {
        return super.getOAuth2Config();
    }
}
exports.GoogleOAuth2Strategy = GoogleOAuth2Strategy;
//# sourceMappingURL=google-strategy.js.map