"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LinkedInOAuth2Strategy = void 0;
const oauth2_strategy_1 = require("./oauth2-strategy");
const passport_linkedin_oauth2_1 = require("passport-linkedin-oauth2");
class LinkedInOAuth2Strategy extends oauth2_strategy_1.OAuth2BaseStrategy {
    constructor(container, config) {
        super(container, "linkedin", {
            clientID: config.clientID,
            clientSecret: config.clientSecret,
            callbackURL: config.callbackURL,
            scope: config.scope || ["r_liteprofile", "r_emailaddress"]
        });
        const strategyConfig = {
            clientID: this.getOAuth2Config().clientID,
            clientSecret: this.getOAuth2Config().clientSecret,
            callbackURL: this.getOAuth2Config().callbackURL,
            scope: this.getOAuth2Config().scope
        };
        this.strategy = new passport_linkedin_oauth2_1.Strategy(strategyConfig, this.validateCallback.bind(this));
    }
    async validateCallback(accessToken, refreshToken, profile, done) {
        var _a, _b, _c, _d;
        try {
            const transformedProfile = {
                id: profile.id,
                email: ((_b = (_a = profile.emails) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.value) || "",
                first_name: ((_c = profile.name) === null || _c === void 0 ? void 0 : _c.givenName) || "",
                last_name: ((_d = profile.name) === null || _d === void 0 ? void 0 : _d.familyName) || "",
                displayName: profile.displayName || "",
                provider: this.provider,
                metadata: {
                    raw: profile,
                    accessToken,
                    refreshToken
                }
            };
            const authProfile = this.transformToAuthProfile(transformedProfile);
            done(null, authProfile);
        }
        catch (error) {
            done(error);
        }
    }
    authenticateRequest(req) {
        this.strategy.authenticate(req, { session: false });
    }
    async transformProfile(profile) {
        return {
            id: profile.id,
            email: profile.email,
            first_name: profile.firstName,
            last_name: profile.lastName,
            displayName: profile.displayName,
            provider: profile.provider,
            metadata: profile.metadata
        };
    }
    async authenticate(reqOrData) {
        if (reqOrData instanceof Request) {
            const query = reqOrData.query;
            const code = query === null || query === void 0 ? void 0 : query.code;
            if (!code) {
                throw new Error("Authorization code is required");
            }
            const accessToken = await this.getAccessToken(code);
            const linkedInProfile = await this.getProfile(accessToken);
            const transformedProfile = await this.transformProfile(linkedInProfile);
            return {
                provider: this.provider,
                accessToken,
                profile: this.transformToAuthProfile(transformedProfile)
            };
        }
        // Handle direct AuthEventData
        if (!this.isAuthEventData(reqOrData)) {
            throw new Error("Invalid auth data format");
        }
        return reqOrData;
    }
    isAuthEventData(data) {
        return data && typeof data === 'object' && 'provider' in data;
    }
    async getProfile(accessToken) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l;
        const url = "https://api.linkedin.com/v2/me?projection=(id,firstName,lastName,profilePicture(displayImage~:playableStreams))";
        const headers = {
            Authorization: `Bearer ${accessToken}`,
            "cache-control": "no-cache",
            "X-Restli-Protocol-Version": "2.0.0"
        };
        const response = await fetch(url, { headers });
        if (!response.ok) {
            throw new Error("Failed to fetch LinkedIn profile");
        }
        const data = await response.json();
        // Get email in a separate request as it requires different endpoint
        const emailUrl = "https://api.linkedin.com/v2/emailAddress?q=members&projection=(elements*(handle~))";
        const emailResponse = await fetch(emailUrl, { headers });
        const emailData = await emailResponse.json();
        const email = ((_c = (_b = (_a = emailData === null || emailData === void 0 ? void 0 : emailData.elements) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b["handle~"]) === null || _c === void 0 ? void 0 : _c.emailAddress) || "";
        const authProfile = {
            id: data.id,
            email: email,
            firstName: ((_e = (_d = data.firstName) === null || _d === void 0 ? void 0 : _d.localized) === null || _e === void 0 ? void 0 : _e.en_US) || "",
            lastName: ((_g = (_f = data.lastName) === null || _f === void 0 ? void 0 : _f.localized) === null || _g === void 0 ? void 0 : _g.en_US) || "",
            displayName: `${((_j = (_h = data.firstName) === null || _h === void 0 ? void 0 : _h.localized) === null || _j === void 0 ? void 0 : _j.en_US) || ""} ${((_l = (_k = data.lastName) === null || _k === void 0 ? void 0 : _k.localized) === null || _l === void 0 ? void 0 : _l.en_US) || ""}`.trim(),
            provider: this.provider,
            metadata: {
                raw: {
                    ...data,
                    email: emailData
                }
            }
        };
        return authProfile;
    }
    async getAccessToken(code) {
        const config = this.getOAuth2Config();
        const params = new URLSearchParams({
            client_id: config.clientID,
            client_secret: config.clientSecret,
            code,
            grant_type: "authorization_code",
            redirect_uri: config.callbackURL
        });
        const url = "https://www.linkedin.com/oauth/v2/accessToken";
        const response = await fetch(url, {
            method: "POST",
            headers: {
                "Content-Type": "application/x-www-form-urlencoded"
            },
            body: params.toString()
        });
        if (!response.ok) {
            throw new Error("Failed to get access token from LinkedIn");
        }
        const data = await response.json();
        if (!data.access_token) {
            throw new Error("No access token received from LinkedIn");
        }
        return data.access_token;
    }
    async afterAuth(data) {
        return data;
    }
    async onSuccess(data) {
        return {
            ...data,
            metadata: {
                ...data.metadata,
                status: "success"
            }
        };
    }
    async onError(data) {
        return {
            ...data,
            metadata: {
                ...data.metadata,
                status: "error"
            }
        };
    }
}
exports.LinkedInOAuth2Strategy = LinkedInOAuth2Strategy;
//# sourceMappingURL=linkedin-strategy.js.map