"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MicrosoftOAuth2Strategy = void 0;
const oauth2_strategy_1 = require("./oauth2-strategy");
const passport_microsoft_1 = require("passport-microsoft");
class MicrosoftOAuth2Strategy extends oauth2_strategy_1.OAuth2BaseStrategy {
    constructor(container, config) {
        super(container, "microsoft", {
            clientID: config.clientID,
            clientSecret: config.clientSecret,
            callbackURL: config.callbackURL,
            scope: config.scope || ["user.read"]
        });
        this.tenant = config.tenant || "common";
        const strategyConfig = {
            clientID: this.getOAuth2Config().clientID,
            clientSecret: this.getOAuth2Config().clientSecret,
            callbackURL: this.getOAuth2Config().callbackURL,
            scope: this.getOAuth2Config().scope,
            tenant: this.tenant
        };
        this.strategy = new passport_microsoft_1.Strategy(strategyConfig, this.validateCallback.bind(this));
    }
    async validateCallback(accessToken, refreshToken, profile, done) {
        var _a, _b, _c, _d;
        try {
            const transformedProfile = {
                id: profile.id,
                email: ((_b = (_a = profile.emails) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.value) || "",
                first_name: ((_c = profile.name) === null || _c === void 0 ? void 0 : _c.givenName) || "",
                last_name: ((_d = profile.name) === null || _d === void 0 ? void 0 : _d.familyName) || "",
                displayName: profile.displayName || "",
                provider: this.provider,
                metadata: {
                    raw: profile,
                    accessToken,
                    refreshToken
                }
            };
            const authProfile = this.transformToAuthProfile(transformedProfile);
            done(null, authProfile);
        }
        catch (error) {
            done(error);
        }
    }
    authenticateRequest(req) {
        this.strategy.authenticate(req, { session: false });
    }
    async transformProfile(profile) {
        return {
            id: profile.id,
            email: profile.email,
            first_name: profile.firstName,
            last_name: profile.lastName,
            displayName: profile.displayName,
            provider: profile.provider,
            metadata: profile.metadata
        };
    }
    async authenticate(reqOrData) {
        if (reqOrData instanceof Request) {
            const query = reqOrData.query;
            const code = query === null || query === void 0 ? void 0 : query.code;
            if (!code) {
                throw new Error("Authorization code is required");
            }
            const accessToken = await this.getAccessToken(code);
            const microsoftProfile = await this.getProfile(accessToken);
            const transformedProfile = await this.transformProfile(microsoftProfile);
            return {
                provider: this.provider,
                accessToken,
                profile: this.transformToAuthProfile(transformedProfile)
            };
        }
        // Handle direct AuthEventData
        if (!this.isAuthEventData(reqOrData)) {
            throw new Error("Invalid auth data format");
        }
        return reqOrData;
    }
    isAuthEventData(data) {
        return data && typeof data === 'object' && 'provider' in data;
    }
    async getProfile(accessToken) {
        const url = "https://graph.microsoft.com/v1.0/me";
        const headers = {
            Authorization: `Bearer ${accessToken}`
        };
        const response = await fetch(url, { headers });
        if (!response.ok) {
            throw new Error("Failed to fetch Microsoft profile");
        }
        const data = await response.json();
        const authProfile = {
            id: data.id,
            email: data.mail || data.userPrincipalName || "",
            firstName: data.givenName || "",
            lastName: data.surname || "",
            displayName: data.displayName || "",
            provider: this.provider,
            metadata: {
                raw: data
            }
        };
        return authProfile;
    }
    async getAccessToken(code) {
        const config = this.getOAuth2Config();
        const params = new URLSearchParams({
            client_id: config.clientID,
            client_secret: config.clientSecret,
            code,
            grant_type: "authorization_code",
            redirect_uri: config.callbackURL
        });
        const url = `https://login.microsoftonline.com/${this.tenant}/oauth2/v2.0/token`;
        const response = await fetch(url, {
            method: "POST",
            headers: {
                "Content-Type": "application/x-www-form-urlencoded"
            },
            body: params.toString()
        });
        if (!response.ok) {
            throw new Error("Failed to get access token from Microsoft");
        }
        const data = await response.json();
        if (!data.access_token) {
            throw new Error("No access token received from Microsoft");
        }
        return data.access_token;
    }
    async afterAuth(data) {
        return data;
    }
    async onSuccess(data) {
        return {
            ...data,
            metadata: {
                ...data.metadata,
                status: "success"
            }
        };
    }
    async onError(data) {
        return {
            ...data,
            metadata: {
                ...data.metadata,
                status: "error"
            }
        };
    }
}
exports.MicrosoftOAuth2Strategy = MicrosoftOAuth2Strategy;
//# sourceMappingURL=microsoft-strategy.js.map