"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthWorkflow = exports.finalizeStep = exports.createOrUpdateUserStep = exports.transformProfileStep = exports.authenticateStep = exports.beforeAuthenticateStep = void 0;
const workflows_sdk_1 = require("@medusajs/workflows-sdk");
const auth_hook_1 = require("./auth.hook");
// Step 1: Before Authentication
exports.beforeAuthenticateStep = (0, workflows_sdk_1.createStep)("before-authenticate", async (input) => {
    // Create AuthHook with container
    const authHook = new auth_hook_1.AuthHook({ container: input.container });
    // Initialize with empty data and pass provider in options
    const result = await authHook.beforeAuthenticate({}, { provider: input.provider });
    return new workflows_sdk_1.StepResponse({
        ...result,
        container: input.container,
        provider: input.provider
    });
});
// Step 2: Authentication
exports.authenticateStep = (0, workflows_sdk_1.createStep)("authenticate", async (input) => {
    const authHook = new auth_hook_1.AuthHook({ container: input.container });
    const result = await authHook.authenticate(input, { provider: input.provider });
    return new workflows_sdk_1.StepResponse({
        ...result,
        container: input.container,
        provider: input.provider
    });
});
// Step 3: Transform Profile
exports.transformProfileStep = (0, workflows_sdk_1.createStep)("transform-profile", async (input) => {
    const authHook = new auth_hook_1.AuthHook({ container: input.container });
    const result = await authHook.transformProfile(input, { provider: input.provider });
    return new workflows_sdk_1.StepResponse({
        ...result,
        container: input.container,
        provider: input.provider
    });
});
// Step 4: Create or Update User
exports.createOrUpdateUserStep = (0, workflows_sdk_1.createStep)("create-or-update-user", async (input) => {
    const authHook = new auth_hook_1.AuthHook({ container: input.container });
    const result = await authHook.createOrUpdateUser(input, { provider: input.provider });
    return new workflows_sdk_1.StepResponse({
        ...result,
        container: input.container,
        provider: input.provider
    });
}, async (input) => {
    var _a;
    if (!((_a = input === null || input === void 0 ? void 0 : input.user) === null || _a === void 0 ? void 0 : _a.id))
        return;
    const userService = input.container.resolve("userService");
    await userService.delete(input.user.id);
});
// Step 5: Finalize
exports.finalizeStep = (0, workflows_sdk_1.createStep)("finalize", async (input) => {
    const authHook = new auth_hook_1.AuthHook({ container: input.container });
    const result = await authHook.finalize(input, { provider: input.provider });
    return new workflows_sdk_1.StepResponse({
        ...result,
        container: input.container,
        provider: input.provider
    });
});
// Main Authentication Workflow
exports.AuthWorkflow = (0, workflows_sdk_1.createWorkflow)("auth", (input) => {
    // Get initial data with proper type assertion
    const initialData = input.data;
    const options = {
        container: input.container,
        provider: input.provider,
        authData: initialData // Use properly typed data
    };
    const beforeAuthResult = (0, exports.beforeAuthenticateStep)(options);
    const authState = beforeAuthResult.data;
    if (!authState)
        throw new Error("Before authenticate step failed");
    const authResult = (0, exports.authenticateStep)(authState);
    if (!(authResult === null || authResult === void 0 ? void 0 : authResult.data))
        throw new Error("Authentication step failed");
    const transformState = authResult.data;
    const transformResult = (0, exports.transformProfileStep)(transformState);
    if (!(transformResult === null || transformResult === void 0 ? void 0 : transformResult.data))
        throw new Error("Transform profile step failed");
    const createUserState = transformResult.data;
    const createUserResult = (0, exports.createOrUpdateUserStep)(createUserState);
    if (!(createUserResult === null || createUserResult === void 0 ? void 0 : createUserResult.data))
        throw new Error("Create user step failed");
    const finalizeState = createUserResult.data;
    const finalizeResult = (0, exports.finalizeStep)(finalizeState);
    if (!(finalizeResult === null || finalizeResult === void 0 ? void 0 : finalizeResult.data))
        throw new Error("Finalize step failed");
    const state = finalizeResult.data;
    return {
        provider: state.provider,
        container: state.container,
        authData: state.data,
        __type: "auth",
        __step__: "finalize"
    };
});
//# sourceMappingURL=auth.js.map