"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthWorkflow = exports.AUTH_WORKFLOW_ID = void 0;
const config_1 = require("../loaders/config");
exports.AUTH_WORKFLOW_ID = "auth";
var AuthWorkflowSteps;
(function (AuthWorkflowSteps) {
    AuthWorkflowSteps["INITIALIZE"] = "initialize";
    AuthWorkflowSteps["AUTHENTICATE"] = "authenticate";
    AuthWorkflowSteps["TRANSFORM_PROFILE"] = "transform_profile";
    AuthWorkflowSteps["CREATE_OR_UPDATE_USER"] = "create_or_update_user";
    AuthWorkflowSteps["FINALIZE"] = "finalize";
})(AuthWorkflowSteps || (AuthWorkflowSteps = {}));
class AuthWorkflow {
    constructor(container) {
        this.container = container;
    }
    getId() {
        return exports.AUTH_WORKFLOW_ID;
    }
    getSteps() {
        return [
            {
                id: AuthWorkflowSteps.INITIALIZE,
                next: [AuthWorkflowSteps.AUTHENTICATE],
                handler: new InitializeStepHandler(this.container),
            },
            {
                id: AuthWorkflowSteps.AUTHENTICATE,
                next: [AuthWorkflowSteps.TRANSFORM_PROFILE],
                handler: new AuthenticateStepHandler(this.container),
            },
            {
                id: AuthWorkflowSteps.TRANSFORM_PROFILE,
                next: [AuthWorkflowSteps.CREATE_OR_UPDATE_USER],
                handler: new TransformProfileStepHandler(this.container),
            },
            {
                id: AuthWorkflowSteps.CREATE_OR_UPDATE_USER,
                next: [AuthWorkflowSteps.FINALIZE],
                handler: new CreateOrUpdateUserStepHandler(this.container),
            },
            {
                id: AuthWorkflowSteps.FINALIZE,
                next: [],
                handler: new FinalizeStepHandler(this.container),
            },
        ];
    }
    async run(data, options) {
        let currentData = data;
        const steps = this.getSteps();
        const context = { options };
        for (const step of steps) {
            try {
                const stepResult = await step.handler.handle(currentData, context);
                // Check if the step returned an AuthWorkflowResult (with success property)
                if (stepResult && 'success' in stepResult) {
                    // Early return if it's a result
                    return stepResult;
                }
                // Otherwise continue with the updated data
                currentData = stepResult;
            }
            catch (error) {
                return {
                    success: false,
                    error: error,
                };
            }
        }
        // If we reach here, we need to transform the data into a result
        return {
            success: true,
            ...currentData
        };
    }
}
exports.AuthWorkflow = AuthWorkflow;
class InitializeStepHandler {
    constructor(container) {
        this.container = container;
    }
    async handle(data, context) {
        try {
            const config = (0, config_1.getConfig)(this.container);
            const provider = context.options.provider;
            const providerConfig = config.providers[provider];
            if (!providerConfig) {
                throw new Error(`Provider ${provider} not configured`);
            }
            return {
                ...data,
                provider,
                metadata: {
                    ...data.metadata,
                    provider,
                    config: providerConfig,
                },
            };
        }
        catch (error) {
            return {
                ...data,
                error: error,
                metadata: {
                    ...data.metadata,
                    provider: context.options.provider,
                },
            };
        }
    }
}
class AuthenticateStepHandler {
    constructor(container) {
        this.container = container;
    }
    async handle(data, context) {
        try {
            if (data.error) {
                return data;
            }
            // This step will be handled by workflow hooks
            return {
                ...data,
                provider: context.options.provider,
            };
        }
        catch (error) {
            return {
                ...data,
                error: error,
            };
        }
    }
}
class TransformProfileStepHandler {
    constructor(container) {
        this.container = container;
    }
    async handle(data, context) {
        try {
            if (data.error || !data.profile) {
                return data;
            }
            // This step will be handled by workflow hooks
            return {
                ...data,
                provider: context.options.provider,
            };
        }
        catch (error) {
            return {
                ...data,
                error: error,
            };
        }
    }
}
class CreateOrUpdateUserStepHandler {
    constructor(container) {
        this.container = container;
    }
    async handle(data, context) {
        try {
            if (data.error || !data.profile) {
                return data;
            }
            // This step will be handled by workflow hooks
            return {
                ...data,
                provider: context.options.provider,
            };
        }
        catch (error) {
            return {
                ...data,
                error: error,
            };
        }
    }
}
class FinalizeStepHandler {
    constructor(container) {
        this.container = container;
    }
    async handle(data, _context) {
        if (data.error) {
            return {
                success: false,
                error: data.error,
            };
        }
        if (!data.user) {
            return {
                success: false,
                error: new Error("No user found after authentication"),
            };
        }
        return {
            success: true,
            user: data.user,
        };
    }
}
//# sourceMappingURL=auth.workflow.js.map